<?php

class ControllerExtensionPaymentDolyame extends Controller
{
	private $error  = array();
	private $prefix = '';

	public function __construct($registry)
	{
		$this->prefix = $this->is23() ? '' : 'payment_';
		parent::__construct($registry);
	}

	private function is23()
	{
		return version_compare(VERSION, '3.0.0.0') == -1;
	}

	private function getTokenKey()
	{
		return $this->is23() ? 'token' : 'user_token';
	}

	public function index()
	{
		$data = [];
		$this->load->language('extension/payment/dolyame');

		$this->document->setTitle($this->language->get('heading_title'));

		$this->load->model('setting/setting');

		$tokenKey   = $this->getTokenKey();
		$returnPath = $this->is23() ? 'extension/extension' : 'marketplace/extension';

		if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
			$this->model_setting_setting->editSetting($this->prefix . 'dolyame', $this->request->post);

			$this->session->data['success'] = $this->language->get('text_success');
			$this->response->redirect($this->url->link($returnPath, $tokenKey . '=' . $this->session->data[$tokenKey] . '&type=payment', true));
		}

		$data['heading_title'] = $this->language->get('heading_title');

		$data['text_enabled']   = $this->language->get('text_enabled');
		$data['help_total']     = $this->language->get('help_total');
		$data['help_key_path']  = $this->language->get('help_key_path');
		$data['help_cert_path'] = $this->language->get('help_cert_path');
		$data['text_disabled']  = $this->language->get('text_disabled');
		$data['text_all_zones'] = $this->language->get('text_all_zones');

		$data['entry_order_status']        = $this->language->get('entry_order_status');
		$data['entry_order_refund_status'] = $this->language->get('entry_order_refund_status');
		$data['entry_geo_zone']            = $this->language->get('entry_geo_zone');

		$data['text_edit'] = $this->language->get('text_edit');

		$data['button_save']   = $this->language->get('button_save');
		$data['button_cancel'] = $this->language->get('button_cancel');

		if (isset($this->error['warning'])) {
			$data['error_warning'] = $this->error['warning'];
		} else {
			$data['error_warning'] = '';
		}

		if (isset($this->error['paymentname'])) {
			$data['error_paymentname'] = $this->error['paymentname'];
		} else {
			$data['error_paymentname'] = '';
		}

		if (isset($this->error['key_path'])) {
			$data['error_key_path'] = $this->error['key_path'];
		} else {
			$data['error_key_path'] = '';
		}

		if (isset($this->error['cert_path'])) {
			$data['error_cert_path'] = $this->error['cert_path'];
		} else {
			$data['error_cert_path'] = '';
		}

		$data['breadcrumbs'] = array();

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_home'),
			'href' => $this->url->link('common/dashboard', $tokenKey . '=' . $this->session->data[$tokenKey], true),
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('text_extension'),
			'href' => $this->url->link($returnPath, $tokenKey . '=' . $this->session->data[$tokenKey] . '&type=payment', true),
		);

		$data['breadcrumbs'][] = array(
			'text' => $this->language->get('heading_title'),
			'href' => $this->url->link('extension/payment/dolyame', $tokenKey . '=' . $this->session->data[$tokenKey], true),
		);

		$data['action'] = $this->url->link('extension/payment/dolyame', $tokenKey . '=' . $this->session->data[$tokenKey], true);

		$data['cancel'] = $this->url->link($returnPath, $tokenKey . '=' . $this->session->data[$tokenKey] . '&type=payment', true);

		$data['vat_list'] = array(
			'none'   => $this->language->get('text_vat_none'),
			'vat0'   => $this->language->get('text_vat_vat0'),
			'vat5'   => $this->language->get('text_vat_vat5'),
			'vat7'   => $this->language->get('text_vat_vat7'),
			'vat10'  => $this->language->get('text_vat_vat10'),
			'vat20'  => $this->language->get('text_vat_vat20'),
			'vat105' => $this->language->get('text_vat_vat105'),
			'vat107' => $this->language->get('text_vat_vat107'),
			'vat110' => $this->language->get('text_vat_vat110'),
			'vat120' => $this->language->get('text_vat_vat120'),
		);

		$data['option_list'] = array(
			0 => $this->language->get('text_option_no'),
			1 => $this->language->get('text_option_yes'),
		);

		$data['method_list'] = array(
			'full_prepayment' => $this->language->get('text_pm_full_prepayment'),
			'full_payment'    => $this->language->get('text_pm_full_payment'),
		);

		$data['object_list'] = array(
			'commodity'             => $this->language->get('text_po_commodity'),
			'excise'                => $this->language->get('text_po_excise'),
			'job'                   => $this->language->get('text_po_job'),
			'service'               => $this->language->get('text_po_service'),
			'gambling_bet'          => $this->language->get('text_po_gambling_bet'),
			'gambling_prize'        => $this->language->get('text_po_gambling_prize'),
			'lottery'               => $this->language->get('text_po_lottery'),
			'lottery_prize'         => $this->language->get('text_po_lottery_prize'),
			'intellectual_activity' => $this->language->get('text_po_intellectual_activity'),
			'payment'               => $this->language->get('text_po_payment'),
			'agent_commission'      => $this->language->get('text_po_agent_commission'),
			'composite'             => $this->language->get('text_po_composite'),
			'another'               => $this->language->get('text_po_another'),
		);

		$settings = [
			'paymentname'             => $this->language->get('text_paymentname'),
			'login'                   => '',
			'password'                => '',
			'cert_path'               => '',
			'key_path'                => '',
			'fiscalisation'           => 0,
			'product_vat'             => 'none',
			'delivery_vat'            => 'none',
			'payment_method'          => 'full_prepayment',
			'payment_object'          => 'commodity',
			'delivery_payment_object' => 'service',
			'ffd120'                  => 0,
			'prefix'                  => '',
			'total'                   => '',
			'logging'                 => 0,
			'order_status_id'         => '5', //Complete
			'order_refund_status_id'  => '11', //Refunded
			'geo_zone_id'             => 0,
			'status'                  => 0,
			'sort_order'              => '',
		];

		foreach ($settings as $key => $default) {
			$data['entry_' . $key] = $this->language->get('entry_' . $key);

			if (isset($this->request->post[$this->prefix . 'dolyame_' . $key])) {
				$data[$this->prefix . 'dolyame_' . $key] = $this->request->post[$this->prefix . 'dolyame_' . $key];
			} elseif ($this->config->has($this->prefix . 'dolyame_' . $key)) {
				$data[$this->prefix . 'dolyame_' . $key] = $this->config->get($this->prefix . 'dolyame_' . $key);
			} else {
				$data[$this->prefix . 'dolyame_' . $key] = $default;
			}
		}

		$this->load->model('localisation/order_status');

		$data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();

		$this->load->model('localisation/geo_zone');

		$data['geo_zones'] = $this->model_localisation_geo_zone->getGeoZones();

		$data['header']      = $this->load->controller('common/header');
		$data['column_left'] = $this->load->controller('common/column_left');
		$data['footer']      = $this->load->controller('common/footer');

		$this->response->setOutput($this->load->view('extension/payment/dolyame', $data));
	}

	protected function validate()
	{
		if (!$this->user->hasPermission('modify', 'extension/payment/dolyame')) {
			$this->error['warning'] = $this->language->get('error_permission');
		}

		if (!$this->request->post[$this->prefix . 'dolyame_paymentname']) {
			$this->error['paymentname'] = $this->language->get('error_paymentname');
		}

		$certPath = isset($this->request->post[$this->prefix . 'dolyame_cert_path']) ? $this->request->post[$this->prefix . 'dolyame_cert_path'] : '';
		$certPath = DIR_APPLICATION . '../' . $certPath;

		if (!file_exists($certPath)) {
			$this->error['cert_path'] = $this->language->get('error_cert_path');
		}

		$keyPath = isset($this->request->post[$this->prefix . 'dolyame_key_path']) ? $this->request->post[$this->prefix . 'dolyame_key_path'] : '';
		$keyPath = DIR_APPLICATION . '../' . $keyPath;

		if (!file_exists($keyPath)) {
			$this->error['key_path'] = $this->language->get('error_key_path');
		}

		return !$this->error;
	}

	public function install()
	{
		$this->db->query("
			CREATE TABLE IF NOT EXISTS `" . DB_PREFIX . "dolyame` (
				`order_id` int(11) NOT NULL,
				`status` varchar(20)  NOT NULL,
				`items` text NOT NULL,
				`receipt` text NULL,
				`date` datetime NOT NULL,
				`amount` decimal(10,2) NOT NULL,
				`refund_id` varchar(36) DEFAULT '',
				`prepaid` decimal(10,2) DEFAULT '0',
				`payment_schedule` text NULL,
				PRIMARY KEY (`order_id`)
		)  ENGINE=MyISAM DEFAULT CHARSET=utf8 COLLATE=utf8_general_ci ");

		$this->model_user_user_group->addPermission($this->user->getId(), 'access', 'extension/payment/dolyame/transactions');
		$this->model_user_user_group->addPermission($this->user->getId(), 'modify', 'extension/payment/dolyame/transactions');
	}

	public function uninstall()
	{
		$this->db->query(' drop table IF EXISTS `' . DB_PREFIX . 'dolyame`');
	}
}
